package com.huiju.estateagents.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.entity.SysAdvancedConfigRelation;
import com.huiju.estateagents.service.ISysAdvancedConfigRelationService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
    * 高级配置对应关系表  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2020-02-26
 */
@RestController
@RequestMapping("/")
public class SysAdvancedConfigRelationController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysAdvancedConfigRelationController.class);

    @Autowired
    public ISysAdvancedConfigRelationService iSysAdvancedConfigRelationService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/sysAdvancedConfigRelation",method= RequestMethod.GET)
    public ResponseBean sysAdvancedConfigRelationList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                                      @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<SysAdvancedConfigRelation> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<SysAdvancedConfigRelation> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<SysAdvancedConfigRelation> result = iSysAdvancedConfigRelationService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("sysAdvancedConfigRelationList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param sysAdvancedConfigRelation 实体对象
     * @return
     */
    @RequestMapping(value="/sysAdvancedConfigRelation",method= RequestMethod.POST)
    public ResponseBean sysAdvancedConfigRelationAdd(@RequestBody SysAdvancedConfigRelation sysAdvancedConfigRelation){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iSysAdvancedConfigRelationService.save(sysAdvancedConfigRelation)){
                responseBean.addSuccess(sysAdvancedConfigRelation);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("sysAdvancedConfigRelationAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/sysAdvancedConfigRelation/{id}", method= RequestMethod.DELETE)
    public ResponseBean sysAdvancedConfigRelationDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iSysAdvancedConfigRelationService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("sysAdvancedConfigRelationDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysAdvancedConfigRelation 实体对象
     * @return
     */
    @RequestMapping(value="/sysAdvancedConfigRelation/{id}",method= RequestMethod.PUT)
    public ResponseBean sysAdvancedConfigRelationUpdate(@PathVariable Integer id,
                                        @RequestBody SysAdvancedConfigRelation sysAdvancedConfigRelation){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iSysAdvancedConfigRelationService.updateById(sysAdvancedConfigRelation)){
                responseBean.addSuccess(sysAdvancedConfigRelation);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("sysAdvancedConfigRelationUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/sysAdvancedConfigRelation/{id}",method= RequestMethod.GET)
    public ResponseBean sysAdvancedConfigRelationGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iSysAdvancedConfigRelationService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("sysAdvancedConfigRelationDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
