package com.huiju.estateagents.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.entity.TaAdvancedController;
import com.huiju.estateagents.service.ITaAdvancedControllerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
    * 高级配置控制表  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2020-02-26
 */
@RestController
@RequestMapping("/api")
public class TaAdvancedControllerController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaAdvancedControllerController.class);

    @Autowired
    public ITaAdvancedControllerService iTaAdvancedControllerService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taAdvancedController",method= RequestMethod.GET)
    public ResponseBean taAdvancedControllerList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                                 @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaAdvancedController> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaAdvancedController> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaAdvancedController> result = iTaAdvancedControllerService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taAdvancedControllerList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taAdvancedController 实体对象
     * @return
     */
    @RequestMapping(value="/taAdvancedController",method= RequestMethod.POST)
    public ResponseBean taAdvancedControllerAdd(@RequestBody TaAdvancedController taAdvancedController){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaAdvancedControllerService.save(taAdvancedController)){
                responseBean.addSuccess(taAdvancedController);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taAdvancedControllerAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taAdvancedController/{id}", method= RequestMethod.DELETE)
    public ResponseBean taAdvancedControllerDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaAdvancedControllerService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taAdvancedControllerDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taAdvancedController 实体对象
     * @return
     */
    @RequestMapping(value="/taAdvancedController/{id}",method= RequestMethod.PUT)
    public ResponseBean taAdvancedControllerUpdate(@PathVariable Integer id,
                                        @RequestBody TaAdvancedController taAdvancedController){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaAdvancedControllerService.updateById(taAdvancedController)){
                responseBean.addSuccess(taAdvancedController);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taAdvancedControllerUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taAdvancedController/{id}",method= RequestMethod.GET)
    public ResponseBean taAdvancedControllerGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaAdvancedControllerService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taAdvancedControllerDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
