package com.huiju.estateagents.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.ExtendContent;
import com.huiju.estateagents.entity.TaLiveActivity;
import com.huiju.estateagents.service.IExtendContentService;
import com.huiju.estateagents.service.ITaLiveActivityService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.net.ssl.HttpsURLConnection;
import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author jobob
 * @since 2020-02-20
 */
@RestController
@RequestMapping("/api")
public class TaLiveActivityController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaLiveActivityController.class);

    @Autowired
    public ITaLiveActivityService iTaLiveActivityService;

    @Autowired
    public IExtendContentService iExtendContentService;

    /**
     * 分页查询列表
     *
     * @param pageNum
     * @param pageSize
     * @param liveActivityTitle 直播活动标题
     * @param cityId            所属城市
     * @param buildingId        所属楼盘
     * @param liveDetailType    详情类型 0是模板，1是自定义
     * @param status            状态
     * @param request
     * @return
     */
    @RequestMapping(value = "/admin/taLiveActivity", method = RequestMethod.GET)
    public ResponseBean taLiveActivityList(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                           @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                           String liveActivityTitle, String cityId, String buildingId, Integer status,
                                           HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            IPage<TaLiveActivity> result = iTaLiveActivityService.listLiveActivityByCondition(pageNum, pageSize, getOrgId(request),
                    liveActivityTitle, cityId, buildingId, status, getTaPersonBuildingListByUserId(request));
            responseBean.addSuccess(result);

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taLiveActivityList -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     *
     * @param taLiveActivity 实体对象
     * @return
     */
    @RequestMapping(value = "/admin/taLiveActivity", method = RequestMethod.POST)
    public ResponseBean taLiveActivityAdd(@RequestBody TaLiveActivity taLiveActivity, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            Integer orgId = getOrgId(request);
            Integer userId = getUserId(request);
            taLiveActivity.setOrgId(orgId);
            taLiveActivity.setCreateUser(userId.toString());
            taLiveActivity.setUpdateUser(userId.toString());
            responseBean = iTaLiveActivityService.taLiveActivityAdd(taLiveActivity);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taLiveActivityAdd -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     */
    @ResponseBody
    @RequestMapping(value = "/admin/delTaLiveActivity", method = RequestMethod.PUT)
    public ResponseBean taLiveActivityDelete(@RequestBody List<TaLiveActivity> taLiveActivity, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            List<Integer> liveActivityIds = new ArrayList<>();
            for (TaLiveActivity activity : taLiveActivity) {
                liveActivityIds.add(activity.getLiveActivityId());
            }
            QueryWrapper<ExtendContent> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id", getOrgId(request));
            queryWrapper.eq("content_type", CommConstant.LIVE);
            queryWrapper.in("target_id", liveActivityIds);
            List<ExtendContent> list = iExtendContentService.list(queryWrapper);
            list.forEach(e -> {
                e.setStatus(CommConstant.NOT_PUBLISH);
            });
            if (list.size() > 0) {
                iExtendContentService.updateBatchById(list);
            }

            responseBean = iTaLiveActivityService.delLiveActivityList(getOrgId(request), taLiveActivity);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taLiveActivityDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     *
     * @param id             实体ID
     * @param taLiveActivity 实体对象
     * @return
     */
    @RequestMapping(value = "/admin/taLiveActivity/{id}", method = RequestMethod.PUT)
    public ResponseBean taLiveActivityUpdate(@PathVariable Integer id,
                                             @RequestBody TaLiveActivity taLiveActivity, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            taLiveActivity.setLiveActivityId(id);
            taLiveActivity.setUpdateUser(getUserId(request).toString());
            QueryWrapper<ExtendContent> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("org_id", getOrgId(request));
            queryWrapper.eq("content_type", CommConstant.LIVE);
            queryWrapper.eq("target_id", taLiveActivity.getLiveActivityId());
            List<ExtendContent> list = iExtendContentService.list(queryWrapper);
            if (CommConstant.NOT_PUBLISH.equals(taLiveActivity.getStatus())) {
                list.forEach(e -> {
                    e.setStatus(CommConstant.NOT_PUBLISH);
                });
            }
            if (list.size() > 0) {
                iExtendContentService.updateBatchById(list);
            }

            if(!StringUtils.isEmpty(taLiveActivity.getLiveRoomParam())){
                taLiveActivity.setLiveAppPath(taLiveActivity.getLiveRoomParam().replace(".html",""));
            }

            if (iTaLiveActivityService.updateById(taLiveActivity)) {
                responseBean.addSuccess(taLiveActivity);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taLiveActivityUpdate -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/admin/taLiveActivity/{id}", method = RequestMethod.GET)
    public ResponseBean taLiveActivityGet(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaLiveActivityService.getById(id));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taLiveActivityDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 访问量自增
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/wx/addVisitNum", method = RequestMethod.PUT)
    public ResponseBean addVisitNum(@RequestParam("id") String id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean = iTaLiveActivityService.addVisitNum(id);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("addVisitNum -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 获取直播详情
     *
     * @param id
     * @return
     */
    @RequestMapping(value = "/wx/taLiveActivity/{id}", method = RequestMethod.GET)
    public ResponseBean getLiveActivityDetailById(@PathVariable("id") String id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean = iTaLiveActivityService.getLiveActivityDetailById(id, getOrgId(request));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("getLiveActivityDetailById -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 分页查询列表
     *
     * @param pageNum
     * @param pageSize
     * @param liveActivityTitle 直播活动标题
     * @param cityId            所属城市
     * @param buildingId        所属楼盘
     * @param status            状态
     * @param request
     * @return
     */
    @RequestMapping(value = "/wx/taLiveActivity", method = RequestMethod.GET)
    public ResponseBean taLiveActivityListForMiniApp(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                     @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                                     String liveActivityTitle, String cityId, String buildingId, Integer status,
                                                     HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            status = status == null ? 1 : status;
            Integer orgId = getOrgId(request);
            IPage<TaLiveActivity> result = iTaLiveActivityService.listLiveActivityByCondition(pageNum, pageSize, orgId,
                    liveActivityTitle, cityId, buildingId, status, null);
            responseBean.addSuccess(result);

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taLiveActivityListForMiniApp -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
