package com.huiju.estateagents.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.entity.TaOrder;
import com.huiju.estateagents.service.ITaOrderService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
    * 订单表  前端控制器
    * </p>
 *
 * @author jobob
 * @since 2020-02-26
 */
@RestController
@RequestMapping("/")
public class TaOrderController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrderController.class);

    @Autowired
    public ITaOrderService iTaOrderService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/taOrder",method= RequestMethod.GET)
    public ResponseBean taOrderList(@RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize){
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
		    IPage<TaOrder> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaOrder> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaOrder> result = iTaOrderService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrderList -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     * @param taOrder 实体对象
     * @return
     */
    @RequestMapping(value="/taOrder",method= RequestMethod.POST)
    public ResponseBean taOrderAdd(@RequestBody TaOrder taOrder){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrderService.save(taOrder)){
                responseBean.addSuccess(taOrder);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrderAdd -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @ResponseBody
    @RequestMapping(value="/taOrder/{id}", method= RequestMethod.DELETE)
    public ResponseBean taOrderDelete(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            if(iTaOrderService.removeById(id)){
                responseBean.addSuccess("success");
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrderDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taOrder 实体对象
     * @return
     */
    @RequestMapping(value="/taOrder/{id}",method= RequestMethod.PUT)
    public ResponseBean taOrderUpdate(@PathVariable Integer id,
                                        @RequestBody TaOrder taOrder){
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaOrderService.updateById(taOrder)){
                responseBean.addSuccess(taOrder);
            }else {
                responseBean.addError("fail");
            }
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrderUpdate -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/taOrder/{id}",method= RequestMethod.GET)
    public ResponseBean taOrderGet(@PathVariable Integer id){
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaOrderService.getById(id));
        }catch (Exception e){
            e.printStackTrace();
            logger.error("taOrderDelete -=- {}",e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
