package com.huiju.estateagents.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.entity.TaPersonBuilding;
import com.huiju.estateagents.entity.TaRaise;
import com.huiju.estateagents.service.ITaRaiseService;
import com.huiju.estateagents.service.ITdCityService;
import io.swagger.models.auth.In;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 认筹表  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2020-02-26
 */
@RestController
@RequestMapping("/api")
public class TaRaiseController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaRaiseController.class);

    @Autowired
    public ITaRaiseService iTaRaiseService;

    @Autowired
    public ITdCityService iTdCityService;

    /**
     * 获取认筹列表
     *
     * @param pageNum
     * @param pageSize
     * @param buildingId
     * @param salesBatchName
     * @param status
     * @param request
     * @return
     */
    @RequestMapping(value = "/admin/listRaiseByCondition", method = RequestMethod.GET)
    public ResponseBean listRaiseByCondition(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                             @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                             String buildingId, String salesBatchName, Integer status,
                                             HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        try {
            Integer orgId = getOrgId(request);
            List<TaPersonBuilding> personBuildingList = getTaPersonBuildingListByUserId(request);

            responseBean = iTaRaiseService.listRaiseByCondition(pageSize, pageNum, salesBatchName, buildingId, orgId, status, personBuildingList);
            logger.info("listRaiseByCondition 返回结果：{}", JSONObject.toJSONString(responseBean));
        } catch (Exception e) {
            logger.error("listRaiseByCondition -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }


    /**
     * 分页查询列表
     *
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value = "/admin/taRaise", method = RequestMethod.GET)
    public ResponseBean taRaiseList(@RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                    @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        ResponseBean responseBean = new ResponseBean();
        try {
            //使用分页插件
            IPage<TaRaise> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TaRaise> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_date");

            IPage<TaRaise> result = iTaRaiseService.page(pg, queryWrapper);
            responseBean.addSuccess(result);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseList -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     *
     * @param taRaise 实体对象
     * @return
     */
    @RequestMapping(value = "/admin/taRaise", method = RequestMethod.POST)
    public ResponseBean taRaiseAdd(@RequestBody TaRaise taRaise, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            // 参数校验
            Integer raisePrice = taRaise.getRaisePrice();
            if (raisePrice != null && raisePrice > CommConstant.MAX_RAISE_PRICE) {
                responseBean.addError("认筹金额最高50000");
                return responseBean;
            }

            Integer orgId = getOrgId(request);

            // 校验批次是否已经认筹过
            QueryWrapper<TaRaise> taRaiseQueryWrapper = new QueryWrapper<>();
            taRaiseQueryWrapper.eq("org_Id", orgId);
            taRaiseQueryWrapper.eq("sales_batch_id", taRaise.getSalesBatchId());
            taRaiseQueryWrapper.eq("status", 1);
            TaRaise tr = iTaRaiseService.getOne(taRaiseQueryWrapper);
            if (tr != null) {
                responseBean.addSuccess("该销售批次已添加过认筹");
                return responseBean;
            }

            taRaise.setOrgId(orgId);
            taRaise.setStatus(CommConstant.IS_PUBLISH);
            taRaise.setCreateDate(LocalDateTime.now());
            if (iTaRaiseService.save(taRaise)) {
                responseBean.addSuccess(taRaise);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseAdd -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量删除
     *
     * @param taRaiseList
     * @param request
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/admin/raise/batchDelete", method = RequestMethod.DELETE)
    public ResponseBean batchDelete(@RequestBody List<TaRaise> taRaiseList, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            Integer orgId = getOrgId(request);
            responseBean = iTaRaiseService.batchDelete(taRaiseList, orgId);

        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     */
    @ResponseBody
    @RequestMapping(value = "/taRaise/{id}", method = RequestMethod.DELETE)
    public ResponseBean taRaiseDelete(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaRaiseService.removeById(id)) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     *
     * @param id      实体ID
     * @param taRaise 实体对象
     * @return
     */
    @RequestMapping(value = "/admin/taRaise/{id}", method = RequestMethod.PUT)
    public ResponseBean taRaiseUpdate(@PathVariable Integer id,
                                      @RequestBody TaRaise taRaise) {
        ResponseBean responseBean = new ResponseBean();
        try {
            taRaise.setRaiseId(id);
            if (iTaRaiseService.updateById(taRaise)) {
                responseBean.addSuccess(taRaise);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseUpdate -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/admin/taRaise/{id}", method = RequestMethod.GET)
    public ResponseBean taRaiseGet(@PathVariable Integer id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean = iTaRaiseService.getRaiseById(id, getOrgId(request));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    @GetMapping("/wx/taRaise")
    public ResponseBean getRaiseProfile(@RequestParam(required = false) Integer salesBatchId,
                                        @RequestParam(required = false) Integer raiseId,
                                        @RequestParam(required = false) Integer raiseRecordId,
                                        @RequestParam(required = false) String personId,
                                        HttpServletRequest request) {
        TaRaise taRaise = iTaRaiseService.getRaiseProfile(raiseId, raiseRecordId, salesBatchId, personId);
        return ResponseBean.success(taRaise);
    }

}
