package com.huiju.estateagents.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.bo.request.HousingRaiseRequestBO;
import com.huiju.estateagents.bo.request.RaiseOrderRequestBO;
import com.huiju.estateagents.bo.request.RaiseRecordRequestBO;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.service.ITaUserService;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.drainage.entity.TaDrainageRecord;
import com.huiju.estateagents.entity.*;
import com.huiju.estateagents.service.*;
import jodd.util.StringUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 认筹单(认筹记录表)  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2020-02-26
 */
@RestController
@RequestMapping("/api")
public class TaRaiseRecordController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaRaiseRecordController.class);

    @Autowired
    public ITaRaiseRecordService iTaRaiseRecordService;

    @Autowired
    public ITaRaiseHouseService taRaiseHouseService;

    @Autowired
    public ITaHousingResourcesService taHousingResourcesService;

    @Autowired
    public ITaOrderService taOrderService;

    @Autowired
    public ITaBuildingImgService taBuildingImgService;

    @Autowired
    public ITaRaiseService taRaiseService;

    @Autowired
    public ITaBuildingApartmentService taBuildingApartmentService;

    @Autowired
    public ITaUserService taUserService;

    @Autowired
    public ITaPersonService taPersonService;

    /**
     * 条件查询房源认筹记录
     *
     * @param requestBO
     * @param pageNumber
     * @param pageSize
     * @param request
     * @return
     */
    @RequestMapping(value = "/admin/listHousingRaiseByCondition", method = RequestMethod.GET)
    public ResponseBean listHousingRaiseByCondition(HousingRaiseRequestBO requestBO,
                                                    @RequestParam(value = "pageNumber", defaultValue = "1") Integer pageNumber,
                                                    @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest request) {
        logger.info("TaRaiseRecordController.listHousingRaiseByCondition 接收参数：bo:{},pageNumber:{},pageSize:{}", JSONObject.toJSONString(requestBO), pageNumber, pageSize);

        ResponseBean responseBean = new ResponseBean();
        try {
            requestBO.setOrgId(getOrgId(request));
            responseBean = iTaRaiseRecordService.listHousingRaiseByCondition(requestBO, pageNumber, pageSize);
            logger.info("TaRaiseRecordController.listHousingRaiseByCondition 返回：{}", JSONObject.toJSONString(responseBean));
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.listHousingRaiseByCondition -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 条件查询认筹单
     *
     * @param recordRequestBO
     * @param pageNumber
     * @param pageSize
     * @param request
     * @return
     */
    @RequestMapping(value = "/admin/listRaiseRecordByCondition", method = RequestMethod.GET)
    public ResponseBean listRaiseRecordByCondition(RaiseRecordRequestBO recordRequestBO,
                                                   @RequestParam(value = "pageNumber", defaultValue = "1") Integer pageNumber,
                                                   @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest request) {
        logger.info("TaRaiseRecordController.listRaiseRecordByCondition 接收参数：bo:{},pageNumber:{},pageSize:{}", JSONObject.toJSONString(recordRequestBO), pageNumber, pageSize);

        ResponseBean responseBean = new ResponseBean();
        try {
            recordRequestBO.setOrgId(getOrgId(request));
            responseBean = iTaRaiseRecordService.listRaiseRecordByCondition(recordRequestBO, pageNumber, pageSize);
            logger.info("TaRaiseRecordController.listRaiseRecordByCondition 返回：{}", JSONObject.toJSONString(responseBean));
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.taRaiseRecordList -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 条件查询缴费单/退费单
     * 备注：
     * 缴费单：payStatus传 paid
     * 退费单：payStatus传 refunded
     *
     * @param orderRequestBO
     * @param pageNumber
     * @param pageSize
     * @param request
     * @return
     */
    @RequestMapping(value = "/admin/listRaiseOrderByCondition", method = RequestMethod.GET)
    public ResponseBean listRaiseOrderByCondition(RaiseOrderRequestBO orderRequestBO,
                                                  @RequestParam(value = "pageNumber", defaultValue = "1") Integer pageNumber,
                                                  @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                                  HttpServletRequest request) {
        logger.info("TaRaiseRecordController.listOrderByCondition 接收参数：bo:{},pageNumber:{},pageSize:{}", JSONObject.toJSONString(orderRequestBO), pageNumber, pageSize);

        ResponseBean responseBean = new ResponseBean();
        try {
            orderRequestBO.setOrgId(84);
            responseBean = iTaRaiseRecordService.listRaiseOrderByCondition(orderRequestBO, pageNumber, pageSize);
            logger.info("TaRaiseRecordController.listOrderByCondition 返回：{}", JSONObject.toJSONString(responseBean));
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.listOrderByCondition -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量作废
     *
     * @param raiseRecord
     * @return
     */
    @RequestMapping(value = "/admin/batchInvalidRaiseRecord", method = RequestMethod.PUT)
    public ResponseBean batchInvalidRaiseRecord(@RequestBody TaRaiseRecord raiseRecord, HttpServletRequest request) {

        logger.info("TaRaiseRecordController.batchInvalidRaiseRecord 接收参数：TaRaiseRecord:{}", JSONObject.toJSONString(raiseRecord));

        ResponseBean responseBean = new ResponseBean();
        try {
            // 校验参数
            if (CollectionUtils.isEmpty(raiseRecord.getTaRaiseRecords()) || StringUtils.isBlank(raiseRecord.getInvalidReason())) {
                responseBean.addError("参数错误");
                return responseBean;
            }

            // 获取用户信息
            TaUser taUser = taUserService.getUserInfoById(getUserId(request), getOrgId(request));

            raiseRecord.getTaRaiseRecords().stream().forEach(record -> {
                if (taUser != null) {
                    record.setUpdateUserName(taUser.getUserName());
                }
                record.setInvalidReason(raiseRecord.getInvalidReason());
                record.setUpdateDate(LocalDateTime.now());
                record.setStatus(0);
            });

            if (iTaRaiseRecordService.updateBatchById(raiseRecord.getTaRaiseRecords())) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.taRaiseRecordList -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量删除
     *
     * @param raiseRecord
     * @return
     */
    @RequestMapping(value = "/admin/batchDeleteRaiseRecord", method = RequestMethod.PUT)
    public ResponseBean batchDeleteRaiseRecord(@RequestBody TaRaiseRecord raiseRecord, HttpServletRequest request) {
        logger.info("TaRaiseRecordController.batchDeleteRaiseRecord 接收参数：{}", JSONObject.toJSONString(raiseRecord));

        ResponseBean responseBean = new ResponseBean();
        try {
            // 参数校验
            if (CollectionUtils.isEmpty(raiseRecord.getTaRaiseRecords())) {
                responseBean.addError("参数错误");
                return responseBean;
            }

            // 获取用户信息
            TaUser taUser = taUserService.getUserInfoById(getUserId(request), getOrgId(request));

            raiseRecord.getTaRaiseRecords().stream().forEach(record -> {
                if (taUser != null) {
                    record.setUpdateUserName(taUser.getUserName());
                }
                record.setStatus(CommConstant.STATUS_DELETE);
                record.setUpdateDate(LocalDateTime.now());
            });

            if (iTaRaiseRecordService.updateBatchById(raiseRecord.getTaRaiseRecords())) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.taRaiseRecordList -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量线下缴费
     *
     * @param raiseRecord
     * @return
     */
    @RequestMapping(value = "/admin/batchPayOfflineRaiseRecord", method = RequestMethod.PUT)
    public ResponseBean batchPayOfflineRaiseRecord(@RequestBody TaRaiseRecord raiseRecord, HttpServletRequest request) {
        logger.info("TaRaiseRecordController.batchPayOfflineRaiseRecord 接收参数：{}", JSONObject.toJSONString(raiseRecord));

        ResponseBean responseBean = new ResponseBean();
        try {
            // 参数校验
            if (CollectionUtils.isEmpty(raiseRecord.getTaRaiseRecords())) {
                responseBean.addError("参数错误");
                return responseBean;
            }

            Integer orgId = getOrgId(request);
            TaRaiseRecord tr;
            QueryWrapper<TaRaiseRecord> queryWrapper;
            for (TaRaiseRecord record : raiseRecord.getTaRaiseRecords()) {
                queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("org_id", orgId);
                queryWrapper.eq("raise_record_id", record.getRaiseRecordId());
                tr = iTaRaiseRecordService.getOne(queryWrapper);
                if (tr == null) {
                    responseBean.addError(String.format("未找到认筹单编号为%s的记录", record.getRaiseRecordId()));
                    return responseBean;
                }
                if (-1 == tr.getStatus()) {
                    responseBean.addError(String.format("认筹单编号为%s的记录已作废，请重新选择", record.getRaiseRecordId()));
                    return responseBean;
                }
                if (!CommConstant.PAY_STATUS_UNPAID.equals(tr.getPayStatus())) {
                    responseBean.addError(String.format("认筹单编号为%s的记录已缴费或意图款，请重新选择", record.getRaiseRecordId()));
                    return responseBean;
                }
            }

            // 获取用户信息
            TaUser taUser = taUserService.getUserInfoById(getUserId(request), orgId);

            // 设置缴费状态为已缴费
            raiseRecord.getTaRaiseRecords().stream().forEach(record -> {
                if (taUser != null) {
                    record.setUpdateUserName(taUser.getUserName());
                }
                record.setPayStatus(CommConstant.PAY_STATUS_PAID);
                record.setUpdateDate(LocalDateTime.now());
            });

            if (iTaRaiseRecordService.updateBatchById(raiseRecord.getTaRaiseRecords())) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.batchPayOfflineRaiseRecord -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 批量线下退费
     *
     * @param raiseRecord
     * @param refundReason    退款原因
     * @return
     */
    @RequestMapping(value = "/admin/batchRefundOfflineRaiseRecord", method = RequestMethod.PUT)
    public ResponseBean batchRefundOfflineRaiseRecord(@RequestBody TaRaiseRecord raiseRecord, String refundReason,
                                                      HttpServletRequest request) {
        logger.info("TaRaiseRecordController.batchRefundOfflineRaiseRecord 接收参数：{},refundReason:{}", JSONObject.toJSONString(raiseRecord), refundReason);

        ResponseBean responseBean = new ResponseBean();
        try {
            // 参数校验
            if (CollectionUtils.isEmpty(raiseRecord.getTaRaiseRecords())) {
                responseBean.addError("参数错误");
                return responseBean;
            }

            Integer orgId = getOrgId(request);
            TaRaiseRecord tr;
            QueryWrapper<TaRaiseRecord> queryWrapper;
            for (TaRaiseRecord record : raiseRecord.getTaRaiseRecords()) {
                queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("org_id", orgId);
                queryWrapper.eq("raise_record_id", record.getRaiseRecordId());
                tr = iTaRaiseRecordService.getOne(queryWrapper);
                if (tr == null) {
                    responseBean.addError(String.format("未找到认筹单编号为%s的记录", record.getRaiseRecordId()));
                    return responseBean;
                }
                if (-1 == tr.getStatus()) {
                    responseBean.addError(String.format("认筹单编号为%s的记录已作废，请重新选择", record.getRaiseRecordId()));
                    return responseBean;
                }
                if (CommConstant.PAY_STATUS_UNPAID.equals(tr.getPayStatus())) {
                    responseBean.addError(String.format("认筹单编号为%s的未缴费，请重新选择", record.getRaiseRecordId()));
                    return responseBean;
                }
                if (CommConstant.PAY_STATUS_REFUNDED.equals(tr.getPayStatus())) {
                    responseBean.addError(String.format("认筹单编号为%s的记录已退费，请重新选择", record.getRaiseRecordId()));
                    return responseBean;
                }
            }

            // 获取用户信息
            TaUser taUser = taUserService.getUserInfoById(getUserId(request), orgId);

            raiseRecord.getTaRaiseRecords().stream().forEach(record -> {
                if (taUser != null) {
                    record.setUpdateUserName(taUser.getUserName());
                }
                record.setPayStatus(CommConstant.PAY_STATUS_REFUNDED);
                record.setRefundReason(refundReason);
                record.setUpdateDate(LocalDateTime.now());
            });

            if (iTaRaiseRecordService.updateBatchById(raiseRecord.getTaRaiseRecords())) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            logger.error("TaRaiseRecordController.batchRefundOfflineRaiseRecord -=- {}", e);
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 保存对象
     *
     * @param taRaiseRecord 实体对象
     * @return
     */
    @RequestMapping(value = "/taRaiseRecord", method = RequestMethod.POST)
    public ResponseBean taRaiseRecordAdd(@RequestBody TaRaiseRecord taRaiseRecord) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaRaiseRecordService.save(taRaiseRecord)) {
                responseBean.addSuccess(taRaiseRecord);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseRecordAdd -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id删除对象
     *
     * @param id 实体ID
     */
    @ResponseBody
    @RequestMapping(value = "/taRaiseRecord/{id}", method = RequestMethod.DELETE)
    public ResponseBean taRaiseRecordDelete(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaRaiseRecordService.removeById(id)) {
                responseBean.addSuccess("success");
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseRecordDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 修改对象
     *
     * @param id            实体ID
     * @param taRaiseRecord 实体对象
     * @return
     */
    @RequestMapping(value = "/taRaiseRecord/{id}", method = RequestMethod.PUT)
    public ResponseBean taRaiseRecordUpdate(@PathVariable Integer id,
                                            @RequestBody TaRaiseRecord taRaiseRecord) {
        ResponseBean responseBean = new ResponseBean();
        try {
            if (iTaRaiseRecordService.updateById(taRaiseRecord)) {
                responseBean.addSuccess(taRaiseRecord);
            } else {
                responseBean.addError("fail");
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseRecordUpdate -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 根据id查询对象
     *
     * @param id 实体ID
     */
    @RequestMapping(value = "/taRaiseRecord/{id}", method = RequestMethod.GET)
    public ResponseBean taRaiseRecordGet(@PathVariable Integer id) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaRaiseRecordService.getById(id));
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("taRaiseRecordDelete -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 条件查询认筹单
     *
     * @param request
     * @return
     */
    @RequestMapping(value = "/admin/getUserInfo", method = RequestMethod.GET)
    public ResponseBean getUserInfo(@RequestParam("raiseRecordId") Integer raiseRecordId,
                                    @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                    @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                                    HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaRaiseRecordService.getUserInfo(raiseRecordId, getOrgId(request)));
        } catch (Exception e) {
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 微信端下认筹单
     *
     * @param taRaiseRecord 实体对象
     * @return
     */
    @RequestMapping(value = "/wx/taRaiseRecord", method = RequestMethod.POST)
    public ResponseBean wxRaiseRecordAdd(@RequestBody TaRaiseRecord taRaiseRecord, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            responseBean.addSuccess(iTaRaiseRecordService.saveRaiseInfo(taRaiseRecord, getOrgId(request), getPersonId(request)));
        } catch (Exception e) {
            logger.error("微信端下认筹单 -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }


    /**
     * 微信端获取认筹单详细信息
     *
     * @param raiseRecordId 认筹旦id
     * @return
     */
    @RequestMapping(value = "/wx/taRaiseRecord/{raiseRecordId}", method = RequestMethod.GET)
    public ResponseBean wxGetRaiseRecordById(@PathVariable Integer raiseRecordId, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            //获取认筹单信息
            TaRaiseRecord taRaiseRecord = iTaRaiseRecordService.getById(raiseRecordId);

            //认筹表数据
            TaRaise taRaise = taRaiseService.getById(taRaiseRecord.getRaiseId());
            taRaise.setRaisePrice(taRaise.getRaisePrice());
            //置业顾问信息
            TaUser consultantInfo = taUserService.getById(taRaiseRecord.getUserId());
            if (null != consultantInfo) {
                //职业顾问信息
                TaPerson consultantPerson = taPersonService.getOne(new QueryWrapper<TaPerson>().eq("user_id", taRaiseRecord.getUserId()));
                if (null != consultantPerson) {
                    consultantInfo.setConsultantPersonId(consultantPerson.getPersonId());
                }
            }

            taRaiseRecord.setConsultantInfo(consultantInfo);
            //房源与认筹单关系图
            List<TaRaiseHouse> taRaiseHouseList = taRaiseHouseService.list(new QueryWrapper<TaRaiseHouse>().eq("raise_record_id", taRaiseRecord.getRaiseRecordId()));

            //房源信息
            List<TaHousingResources> houseList = (List<TaHousingResources>) taHousingResourcesService.listByIds(taRaiseHouseList.stream().map(TaRaiseHouse::getHouseId).collect(Collectors.toList()));

            //我的认筹和锁定
            houseList.forEach(e -> {
                taRaiseHouseList.forEach(r -> {
                    //我的认筹
                    if (e.getHouseId().equals(r.getHouseId()) && CommConstant.HOUSE_LOCKING_STATUS_LOCKED.equals(r.getHouseLockingStatus())) {
                        e.setMineLocked("mine");
                    }
                });

                //塞入户型信息
                if (!StringUtils.isEmpty(e.getApartmentId())) {
                    TaBuildingApartment buildingApartment = taBuildingApartmentService.getById(e.getApartmentId());
                    e.setApartmentName(buildingApartment.getApartmentName());
                    e.setInsideArea(buildingApartment.getInsideArea());
                    e.setBuildingArea(buildingApartment.getBuildingArea());
                }
            });

            Map<String, Object> map = new HashMap<>();
            map.put("refundOrder", null);
            //订单信息
            TaOrder taOrder = taOrderService.getById(taRaiseRecord.getOrderId());
            if (null != taOrder) {
                //退费信息
                TaOrder refundOrder = taOrderService.getOne(new QueryWrapper<TaOrder>().eq("parent_order_id", taOrder.getOrderId()));
                map.put("refundOrder", refundOrder);
            }
            map.put("taRaiseRecord", taRaiseRecord);
            map.put("taRaiseHouseList", taRaiseHouseList);
            map.put("houseList", houseList);
            map.put("taOrder", taOrder);

            map.put("taRaise", taRaise);
            responseBean.addSuccess(map);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("微信端获取认筹单详细信息 -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }

    /**
     * 微信端获取认筹单列表信息
     *
     * @return
     */
    @RequestMapping(value = "/wx/taRaiseRecord/list", method = RequestMethod.GET)
    public ResponseBean wxGetRaiseRecordList(@RequestParam(value = "buildingId", required = false) String buildingId,
                                             @RequestParam(value = "salesBtchId", required = false) Integer salesBtchId,
                                             @RequestParam(value = "houseLockingStatus", required = false) String houseLockingStatus,
                                             @RequestParam(value = "pageNumber", defaultValue = "1") Integer pageNumber,
                                             @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();
        try {
            IPage<TaRaiseRecord> pg = new Page<>(pageNumber, pageSize);
            //获取认筹单列表信息
            QueryWrapper<TaRaiseRecord> taRaiseRecordQueryWrapper = new QueryWrapper<>();
            taRaiseRecordQueryWrapper.eq("org_id", getOrgId(request));
            taRaiseRecordQueryWrapper.eq("person_id", getPersonId(request));
            taRaiseRecordQueryWrapper.eq(StringUtils.isNotEmpty(buildingId), "building_id", buildingId);
            taRaiseRecordQueryWrapper.eq(null != salesBtchId, "sales_batch_id", salesBtchId);
            taRaiseRecordQueryWrapper.eq(StringUtils.isNotEmpty(houseLockingStatus), "house_locking_status", houseLockingStatus);
            IPage<TaRaiseRecord> page = iTaRaiseRecordService.page(pg, taRaiseRecordQueryWrapper);
            List<TaRaiseRecord> list = page.getRecords();
            list.forEach(e -> {
                //房源与认筹单关系图
                List<TaRaiseHouse> taRaiseHouseList = taRaiseHouseService.list(new QueryWrapper<TaRaiseHouse>().eq("raise_record_id", e.getRaiseRecordId()));
                //房源信息
                List<TaHousingResources> houseList = (List<TaHousingResources>) taHousingResourcesService.listByIds(taRaiseHouseList.stream().map(TaRaiseHouse::getHouseId).collect(Collectors.toList()));
                //户型图列表
                List<TaBuildingImg> apartMentList = taBuildingImgService.getAparemnetList(houseList.stream().map(TaHousingResources::getApartmentId).collect(Collectors.toList()));
                e.setTaHousingResourcesList(houseList);
                e.setTaHousingResourcesSize(houseList.size());
                e.setApartMentList(apartMentList);
            });

            responseBean.addSuccess(page);
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("微信端获取认筹单列表信息 -=- {}", e.toString());
            responseBean.addError(e.getMessage());
        }
        return responseBean;
    }
}
