package com.huiju.estateagents.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.bo.request.HousingRaiseRequestBO;
import com.huiju.estateagents.bo.request.RaiseOrderRequestBO;
import com.huiju.estateagents.bo.request.RaiseRecordRequestBO;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.service.impl.TaUserServiceImpl;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.entity.*;
import com.huiju.estateagents.mapper.*;
import com.huiju.estateagents.service.ITaRaiseRecordService;
import com.huiju.estateagents.service.ITaUvService;
import jodd.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 认筹单(认筹记录表)  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2020-02-26
 */
@Service
public class TaRaiseRecordServiceImpl extends ServiceImpl<TaRaiseRecordMapper, TaRaiseRecord> implements ITaRaiseRecordService {

    private Logger logger = LoggerFactory.getLogger(TaRaiseRecordServiceImpl.class);

    @Autowired
    private TaRaiseRecordMapper taRaiseRecordMapper;

    @Autowired
    private TaRaiseMapper taRaiseMapper;

    @Autowired
    private TaHousingResourcesMapper taHousingResourcesMapper;

    @Autowired
    private TaSalesBatchMapper taSalesBatchMapper;

    @Autowired
    private TaPersonMapper taPersonMapper;

    @Autowired
    private TaRaiseHouseMapper taRaiseHouseMapper;

    @Autowired
    private TaUserServiceImpl taUserService;

    @Override
    public ResponseBean listHousingRaiseByCondition(HousingRaiseRequestBO requestBO, Integer pageNumber, Integer pageSize) {
        logger.info("TaRaiseRecordServiceImpl.listRaiseRecordByCondition 接收参数：bo:{},pageNumber:{},pageSize:{}", JSONObject.toJSONString(requestBO), pageNumber, pageSize);

        ResponseBean responseBean = new ResponseBean();
        IPage<TaRaiseRecord> pg = new Page<>(pageNumber, pageSize);
        responseBean.addSuccess(taRaiseRecordMapper.listHousingRaiseByCondition(pg, requestBO));
        return responseBean;
    }

    @Override
    public ResponseBean listRaiseRecordByCondition(RaiseRecordRequestBO recordRequestBO, Integer pageNumber, Integer pageSize) {
        logger.info("TaRaiseRecordServiceImpl.listRaiseRecordByCondition 接收参数：bo:{},pageNumber:{},pageSize:{}", JSONObject.toJSONString(recordRequestBO), pageNumber, pageSize);

        ResponseBean responseBean = new ResponseBean();
        IPage<TaRaiseRecord> pg = new Page<>(pageNumber, pageSize);
        responseBean.addSuccess(taRaiseRecordMapper.listRaiseRecordByCondition(pg, recordRequestBO));
        return responseBean;
    }

    @Override
    public ResponseBean listRaiseOrderByCondition(RaiseOrderRequestBO orderRequestBO, Integer pageNumber, Integer pageSize) {
        logger.info("TaRaiseRecordServiceImpl.listOrderByCondition 接收参数：bo:{},pageNumber:{},pageSize:{}", JSONObject.toJSONString(orderRequestBO), pageNumber, pageSize);

        ResponseBean responseBean = new ResponseBean();
        IPage<TaRaiseRecord> pg = new Page<>(pageNumber, pageSize);
        responseBean.addSuccess(taRaiseRecordMapper.listRaiseOrderByCondition(pg, orderRequestBO));
        return responseBean;
    }

    /**
     * 小程序端保存认筹单信息
     *
     * @param taRaiseRecord
     * @param orgId
     * @param personId
     * @return
     */
    @Override
    public TaRaiseRecord saveRaiseInfo(TaRaiseRecord taRaiseRecord, Integer orgId, String personId) throws Exception {
        List<Integer> houseList = taRaiseRecord.getHouseList();
        if (houseList.size() < 1) {
            throw new Exception("请选择房源！");
        }
        //获取认筹信息
        TaRaise taRaise = taRaiseMapper.selectById(taRaiseRecord.getRaiseId());
        //如果是自动锁房就要判断里面的房源有没有被锁住
        if (taRaise.getHouseLockingType().equals(CommConstant.HOUSE_LOCKING_TYPE_AUTO)) {
            if (houseList.size() > 0) {
                throw new Exception("此认筹为自动锁房只能选择一个房源！");
            }
            //获取房源详细信息
            TaHousingResources taHousingResources = taHousingResourcesMapper.selectById(houseList.get(0));
            if (null == taHousingResources) {
                throw new Exception("此房源不存在！");
            }
            if (taHousingResources.getHouseLockingStatus().equals(CommConstant.HOUSE_LOCKING_STATUS_LOCKED)) {
                throw new Exception("此房源已被锁定！");
            }
            //检验成功的话先锁定房源
            lockedHouseResources(taHousingResources);
        }
        //插入认筹单
        saveRaiseRecord(taRaiseRecord, orgId, personId, taRaise);

        return taRaiseRecord;
    }

    @Override
    public TaRaiseRecord getUserInfo(Integer raiseRecordId, Integer orgId) {
        return taRaiseRecordMapper.getUserInfo(raiseRecordId, orgId);
    }

    @Override
    public Boolean addHouseNum(Integer raiseRecordId, Integer orgId, Integer userId) {

        String updateUser = "";

        // 获取用户信息
        QueryWrapper<TaUser> userQueryWrapper = new QueryWrapper<>();
        userQueryWrapper.eq("user_id", userId);
        userQueryWrapper.ne("status", -1);
        TaUser taUser = taUserService.getOne(userQueryWrapper);
        if (taUser != null) {
            updateUser = taUser.getUserName();
        }
        return taRaiseRecordMapper.addHouseNum(raiseRecordId, orgId, updateUser);
    }

    /**
     * 插入认筹单
     *
     * @param taRaiseRecord
     */
    private void saveRaiseRecord(TaRaiseRecord taRaiseRecord, Integer orgId, String personId, TaRaise taRaise) {
        List<Integer> houseList = taRaiseRecord.getHouseList();
        //销售批次信息
        TaSalesBatch taSalesBatch = taSalesBatchMapper.selectById(taRaiseRecord.getSalesBatchId());
        //获取用户信息

        taRaiseRecord.setOrgId(orgId);
        taRaiseRecord.setBuildingId(taSalesBatch.getBuildingId());
        taRaiseRecord.setPersonId(personId);
        taRaiseRecord.setHouseNum(houseList.size());
        taRaiseRecord.setHouseLockingStatus(CommConstant.HOUSE_LOCKING_STATUS_UNLOCKED);
        if (taRaise.getHouseLockingType().equals(CommConstant.HOUSE_LOCKING_TYPE_AUTO)) {
            taRaiseRecord.setHouseLockingStatus(CommConstant.HOUSE_LOCKING_STATUS_LOCKED);
        }
        taRaiseRecord.setStatus(CommConstant.STATUS_NORMAL);
        taRaiseRecord.setPayStatus(CommConstant.PAY_STATUS_UNPAID);
        taRaiseRecord.setPayType(taRaise.getPayType());
        taRaiseRecord.setCreateDate(LocalDateTime.now());
        //插入认筹单信息
        taRaiseRecordMapper.insert(taRaiseRecord);

        //插入房源与认筹单关系表
        TaPerson taPerson = taPersonMapper.selectById(personId);
        houseList.forEach(e -> {
            TaRaiseHouse taRaiseHouse = new TaRaiseHouse();
            taRaiseHouse.setHouseId(e);
            taRaiseHouse.setCreateDate(LocalDateTime.now());
            if (taRaise.getHouseLockingType().equals(CommConstant.HOUSE_LOCKING_TYPE_AUTO)) {
                taRaiseHouse.setHouseLockingStatus(CommConstant.HOUSE_LOCKING_STATUS_LOCKED);
            }
            taRaiseHouse.setOrgId(orgId);
            taRaiseHouse.setPersonId(personId);
            taRaiseHouse.setPersonName(taPerson.getName());
            taRaiseHouse.setRaiseRecordId(taRaiseRecord.getRaiseRecordId());
            taRaiseHouseMapper.insert(taRaiseHouse);
        });

        //把idcard字断更新进person表
        if (StringUtil.isEmpty(taPerson.getIdcard())){
            taPerson.setIdcard(taRaiseRecord.getIdcard());
            taPersonMapper.updateById(taPerson);
        }

    }

    /**
     * 锁定房源
     *
     * @param taHousingResources
     */
    private synchronized void lockedHouseResources(TaHousingResources taHousingResources) {
        taHousingResources.setHouseLockingStatus(CommConstant.HOUSE_LOCKING_STATUS_LOCKED);
        taHousingResourcesMapper.updateById(taHousingResources);
    }
}
